@@ -3,7 +3,7 @@
 ## mklibsrc.sh --   make the libopts tear-off library source tarball
 ##
 ##  This file is part of AutoGen.
-##  AutoGen Copyright (C) 1992-2015 by Bruce Korb - all rights reserved
+##  AutoGen Copyright (C) 1992-2017 by Bruce Korb - all rights reserved
 ##
 ##  AutoGen is free software: you can redistribute it and/or modify it
 ##  under the terms of the GNU General Public License as published by the
@@ -18,6 +18,7 @@
 ##  You should have received a copy of the GNU General Public License along
 ##  with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+test ${#CDPATH} -gt 0 && CDPATH=''
 top_builddir=`cd $top_builddir ; pwd`
 top_srcdir=`cd $top_srcdir ; pwd`
 
@@ -42,7 +43,7 @@ files='libopts.c gettext.h parse-duration.c parse-duration.h
     fgrep '#include' libopts.c | \
        sed -e 's,"$,,;s,#.*",,' )
 
-for f in ${files} intprops.h
+for f in ${files} intprops.h verify.h
 do
   test -f ${f} &&
     cp -f ${f} ${tagd}/${f} && continue
@@ -52,6 +53,8 @@ do
 
   test -f ${top_srcdir}/${f} &&
     cp -f ${top_srcdir}/${f} ${tagd}/${f} && continue
+
+  die "could not locate ${f} to copy into tarball"
 done
 
 cp -f ${top_srcdir}/pkg/libopts/COPYING.* ${tagd}/.
@@ -107,13 +110,23 @@ EOMakefile
     | ${CLexe} -I4 --spread=1 --line-sep="  \\"
 } > Makefile.am
 
-gz='gzip --best'
+gz='gzip --best -n'
 sfx=tar.gz
 
 cd ..
 echo ! cd `pwd`
 echo ! tar cvf ${tag}.${sfx} ${tag}
-tar cvf - ${tag} | $gz > ${top_builddir}/autoopts/${tag}.${sfx}
+
+# If we have a SOURCE_DATE_EPOCH *and* tar supports a sort option,
+# then add some fancy options to make tar output repeatable.
+#
+rbopts=""
+[ -z "$SOURCE_DATE_EPOCH" ] \
+    || ! tar --help|grep -q sort= \
+    || rbopts="--sort=name --format=gnu --clamp-mtime --mtime @$SOURCE_DATE_EPOCH"
+
+tar cvf - $rbopts ${tag} | \
+    $gz > ${top_builddir}/autoopts/${tag}.${sfx}
 rm -rf ${tag}
 
 ## Local Variables:
